package Exam;

import org.jetbrains.annotations.NotNull;

import java.util.*;
import java.util.concurrent.ThreadLocalRandom;

public class WeatherStation {
    private static WeatherStation instance;
    private String stationName;
    public List<String> weatherDataHistory;

    private WeatherStation(String stationName) {
        this.stationName = stationName;
        this.weatherDataHistory = new ArrayList<>();
    }

    public static WeatherStation getInstance(String stationName) {
        if (instance == null) {
            instance = new WeatherStation(stationName);
        }
        return instance;
    }

    public String getWeatherData() {
        String currentWeather = "Weather data.\nTemperature: " + ThreadLocalRandom.current().nextInt(-20, 41);
        weatherDataHistory.add(currentWeather);
        return currentWeather;
    }

    public List<String> getWeatherDataHistory() {
        return weatherDataHistory;
    }

    public class WeatherProxy {
        private WeatherStation weatherStation;
        private String stationName;
        private WeatherObserver observer;

        public WeatherProxy(String stationName) {
        this.stationName = stationName;
        }

        private void checkExists() {
            if (weatherStation == null) {
                this.weatherStation = WeatherStation.getInstance(stationName);
            }
        }

        public String getWeatherData() {
            checkExists();

            observer.notify();
            return weatherStation.getWeatherData();
        }

        public List<String> getWeatherDataHistory() {
            checkExists();

            observer.notify();
            return weatherStation.getWeatherDataHistory();
        }


        public WeatherIterator iterator() {
            return new WeatherIterator(this.getWeatherDataHistory());
        }
    }

    public class WeatherObserver {
        private WeatherProxy weatherStation;
        private List<Object> subscribers = new ArrayList<>();

        public WeatherObserver(WeatherProxy weatherStation, Object... subscribers) {
            this.weatherStation = weatherStation;
            this.subscribers.addAll(Arrays.asList(subscribers));
        }

        public void subscribe(Object subscriber) {
            this.subscribers.add(subscriber);
        }

        public void unsubscribe(Object subscriber) {
            this.subscribers.remove(subscriber);
        }

        public void notify() {
            for (Object subscriber : subscribers) {
//                subscriber.update()
            }
        }
    }

    public class WeatherIterator {
        private List<String> weatherDataHistory;
        private int currentPosition = 0;

        public WeatherIterator(List<String> weatherDataHistory) {
            this.weatherDataHistory = weatherDataHistory;
        }

        public boolean hasNext() {
            return currentPosition < weatherDataHistory.size();
        }

        public String getNext() {
            if (!this.hasNext()) {
                return null;
            }

            String nextString = weatherDataHistory.get(currentPosition);
            currentPosition++;

            return nextString;
        }
    }
}

