package MKR2;

import java.util.ArrayList;
import java.util.List;

class Task {
    private String name;
    private String description;
    private boolean completed;

    public Task(String name, String description) {
        this.name = name;
        this.description = description;
        this.completed = false;
    }

    public String getName() {
        return name;
    }

    public String getDescription() {
        return description;
    }

    public boolean isCompleted() {
        return completed;
    }

    public void complete() {
        completed = true;
    }
}

class TaskFactory {
    public static Task createTask(String name, String description) {
        return new Task(name, description);
    }
}

class PriorityTaskDecorator extends Task {
    private int priority;

    public PriorityTaskDecorator(Task task, int priority) {
        super(task.getName(), task.getDescription());
        this.priority = priority;
    }

    public int getPriority() {
        return priority;
    }
}

class TaskManager {
    private static TaskManager instance;
    private List<Task> tasks;
    private List<User> users;

    private TaskManager() {
        tasks = new ArrayList<>();
    }

    public static TaskManager getInstance() {
        if (instance == null) {
            instance = new TaskManager();
        }
        return instance;
    }

    public void subscribe(User user) {
        this.users.add(user);
    }

    private void notify(String message) {
        for (User user : users) {
            user.update(message);
        }
    }

    public void addTask(Task task) {
        tasks.add(task);
        notify("Task added: " + task.getDescription());
    }

    public void completeTask(Task task) {
        task.complete();
        notify("Task completed: " + task.getDescription());
    }

    public List<Task> getTasks() {
        return tasks;
    }
}

class User {
    private String name;

    public User(String name) {
        this.name = name;
    }

    public void update(String message) {
        System.out.println(name + " received notification: " + message);
    }
}

// Examples of use
class Main {
    public static void main(String[] args) {
        // Creating a user and subscribing to the Observer
        User user = new User("User1");
        TaskManager.getInstance().subscribe(user);

        // Creating a task and adding to a task manager
        Task task1 = TaskFactory.createTask("Work", "Do something productive, idk.");
        TaskManager.getInstance().addTask(task1);

        Task task2 = TaskFactory.createTask("Rest", "Do something else, idk.");
        TaskManager.getInstance().addTask(task2);

        // Completing a task
        TaskManager.getInstance().completeTask(task1);

        // Displaying a list of tasks for a user
        System.out.println("User's tasks:");
        for (Task task : TaskManager.getInstance().getTasks()) {
            System.out.println(task.getDescription() + " - Completed: " + task.isCompleted());
        }
    }
}