package labwork3;

/**
 * The labwork3.ImageProxy class is an implementation of the labwork3.Image interface that acts as a proxy for the labwork3.RealImage class.
 * It provides methods to handle image operations such as selecting pixels, drawing pixels, finding colour by coordinates, and displaying the image.
 */
public class ImageProxy implements Image {
    private int width, height;
    private RealImage realImage;

    /**
     * Constructs an labwork3.ImageProxy with the specified width and height.
     *
     * @param width  the width of the image
     * @param height the height of the image
     */
    public ImageProxy(int width, int height) {
        this.width = width;
        this.height = height;
    }

    /**
     * Checks if the realImage is initialized, and initializes it if it's null.
     */
    private void checkRealImage() {
        if (realImage == null) {
            realImage = new RealImage(width, height);
        }
    }

    /**
     * Checks if the provided coordinates are within the image bounds.
     *
     * @param x the x-coordinate
     * @param y the y-coordinate
     */
    private void checkCoordinates(int x, int y) {
        if (x < 0 || x > width || y < 0 || y > height) {
            System.out.println("Coordinates out of image size.");
            System.exit(1);
        }
    }

    /**
     * Selects a pixel at the specified coordinates.
     *
     * @param x the x-coordinate of the pixel
     * @param y the y-coordinate of the pixel
     */
    @Override
    public void selectPixel(int x, int y) {
        checkRealImage();
        checkCoordinates(x, y);

        realImage.selectPixel(x, y);
    }

    /**
     * Draws a pixel at the specified coordinates.
     *
     * @param x the x-coordinate of the pixel
     * @param y the y-coordinate of the pixel
     */
    @Override
    public void drawPixel(int x, int y) {
        checkRealImage();
        checkCoordinates(x, y);

        realImage.drawPixel(x, y);
    }

    /**
     * Finds the colour of the pixel at the specified coordinates.
     *
     * @param x the x-coordinate of the pixel
     * @param y the y-coordinate of the pixel
     */
    @Override
    public void findColourByCoordinates(int x, int y) {
        checkRealImage();
        checkCoordinates(x, y);

        realImage.findColourByCoordinates(x, y);
    }

    /**
     * Displays the image along with its width and height.
     */
    @Override
    public void display() {
        checkRealImage();

        realImage.display();
    }
}
