package labwork5;

import java.util.ArrayList;
import java.util.List;

/**
 * The ConcreteMediator class implements the Mediator interface and manages the communication between InteractiveElements.
 */
public class ConcreteMediator implements Mediator{
    private List<InteractiveElement> interactiveElements;

    public ConcreteMediator() {
        interactiveElements = new ArrayList<>();
    }

    /**
     * Adds an InteractiveElement to the list of managed elements.
     *
     * @param element the InteractiveElement to be added to the mediator
     */
    @Override
    public void addElement(InteractiveElement element) {
        interactiveElements.add(element);
    }

    /**
     * Updates all InteractiveElements except the source with the provided update message.
     *
     * @param source the source InteractiveElement that triggered the update
     * @param update the update message or information
     */
    @Override
    public void update(InteractiveElement source, String update) {
        for (InteractiveElement element : interactiveElements) {
            if (element != source) {
                element.receiveUpdate(update);
            }
        }
    }
}
