package labwork5;

/**
 * The TextField class is an implementation of the InteractiveElement interface representing a text field in a GUI.
 * It can send and receive updates through a Mediator.
 */
public class TextField implements InteractiveElement{

    private String id;
    private Mediator mediator;
    private String text;

    /**
     * Constructs a TextField with a specified ID, associated Mediator, and initial text.
     *
     * @param id       the unique identifier for the text field
     * @param mediator the Mediator to communicate with
     * @param text     the initial text in the text field
     */
    public TextField(String id, Mediator mediator, String text) {
        this.id = id;
        this.mediator = mediator;
        this.text = text;
    }

    /**
     * Retrieves the ID of the text field.
     *
     * @return the ID of the text field
     */
    public String getId() {
        return id;
    }

    /**
     * Sends an update message to the associated Mediator.
     *
     * @param updateMessage the update message to be sent
     */
    @Override
    public void sendUpdate(String updateMessage) {
        mediator.update(this, updateMessage);
    }

    /**
     * Receives an update message from the Mediator and prints a message.
     *
     * @param updateMessage the received update message
     */
    @Override
    public void receiveUpdate(String updateMessage) {
        System.out.printf("%s has has received an update with a message:\"%s\"\n", id, updateMessage);
    }
}
