package labwork7;

/**
 * The Mouse class represents a mouse device and maintains the current state of the pressed button.
 * It provides methods for left and right clicks, delegating the behavior to the current pressed button state.
 */
public class Mouse {
    private PressedButton pressedButton;

    /**
     * Constructs a Mouse with an initial state of "Nothing" (no button pressed).
     */
    public Mouse() {
        this.pressedButton = new Nothing();
    }

    /**
     * Retrieves the currently pressed button state.
     *
     * @return the current pressed button state
     */
    public PressedButton getPressedButton() {
        return pressedButton;
    }

    /**
     * Sets the pressed button state.
     *
     * @param pressedButton the new pressed button state
     */
    public void setPressedButton(PressedButton pressedButton) {
        this.pressedButton = pressedButton;
    }

    /**
     * Handles a left-click event, delegating the behavior to the current pressed button state.
     */
    public void leftClick() {
        pressedButton.leftClick(this);
    }

    /**
     * Handles a right-click event, delegating the behavior to the current pressed button state.
     */
    public void rightClick() {
        pressedButton.rightClick(this);
    }
}
