package labwork8;

import java.util.Arrays;

/**
 * The GameWorld class is an implementation of the GameWorldComponent and GameWorldPrototype interfaces.
 * It represents a composite of GameWorldComponents and can create shallow and deep copies of itself as per Prototype pattern.
 */
public class GameWorld implements GameWorldComponent, GameWorldPrototype {
    private GameWorldComponent[] gameWorldComponents;
    private String name;

    /**
     * Constructs a GameWorld with a specified name and an array of GameWorldComponents.
     *
     * @param name                the name of the game world
     * @param gameWorldComponents the components that make up the game world
     */
    public GameWorld(String name, GameWorldComponent... gameWorldComponents) {
        this.name = name;
        this.gameWorldComponents = gameWorldComponents;
    }

    /**
     * Retrieves the array of GameWorldComponents in the game world.
     *
     * @return the array of GameWorldComponents
     */
    public GameWorldComponent[] getGameWorldComponents() {
        return gameWorldComponents;
    }

    /**
     * Retrieves the name of the game world.
     *
     * @return the name of the game world
     */
    public String getName() {
        return name;
    }

    /**
     * Creates a shallow copy of the GameWorld.
     *
     * @return a shallow copy of the GameWorld
     */
    @Override
    public GameWorld shallowCopy() {
        return new GameWorld(
                name,
                gameWorldComponents
        );
    }

    /**
     * Creates a deep copy of the GameWorld.
     *
     * @return a deep copy of the GameWorld
     */
    @Override
    public GameWorld deepCopy() {
        return new GameWorld(
                new String(name),
                Arrays.copyOf(gameWorldComponents, gameWorldComponents.length)
        );
    }
}
