package labwork9;

/**
 * The Main class configures the App based on the operating system and demonstrates the Abstract Factory pattern.
 */
public class Main {
    /**
     * Configures and creates an App based on the operating system.
     *
     * @return the configured App
     */
    private static App configureApp() {
        GUIFactory guiFactory;

        String osName = System.getProperty("os.name").toLowerCase();
        if (osName.contains("windows")) {
            guiFactory = new WinAPIFactory();
        } else {
            guiFactory = new GTKFactory();
        }

        return new App(guiFactory);
    }

    /**
     * The entry point of the program.
     *
     * @param args the command-line arguments
     */
    public static void main(String[] args) {
        App app = configureApp();
        app.render();
    }
}
